(:~
 : The aim of this module is to display all XML and RDF data that is stored in
 : the bdn-portal. The data is separated into the three different kinds of RDF
 : objects (persons, works, and instances) and the different BdN authors. For
 : each type of RDF object and each TextGrid edition a Bootstrap panel is
 : rendered with a list of all related files.
 :
 : @author Michelle Weidling
 : @version: 1.0
 :)

xquery version "3.1";

module namespace internal-nav="http://bdn-edition.de/ns/internal-nav";

import module namespace bdn-utils="http://bdn-edition.de/ns/utils" at "utils.xqm";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace functx = "http://www.functx.com";


declare namespace tgmd="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#";


declare variable $internal-nav:metacollection := collection($config:sade-meta);
declare variable $internal-nav:aggcollection := collection($config:sade-agg);

declare variable $internal-nav:relevant-aggs := (
    "2228t", (: Nösselt 1 :)
    "229xf", (: Nösselt 2 :)
    "22g24", (: Nösselt 3 :)
    "250b3", (: Griesbach :)
    "257vp", (: Niemeyer :)
    "257vs", (: Leß :)
    "257vx", (: Töllner :)
    "257w0", (: Ernesti :)
    "257w2", (: Bahrdt/Semler :)
    "257w4", (: Sack :)
    "257w7", (: Teller :)
    "257wb"); (: Steinbart :)

(:~
 : Displays the panels for the RDF objects and the XML files (grouped by
 : aggregation).
 :
 : @author Michelle Weidling
 : @return A string indicating the location of the stored HTML file.
 : @version 1.0
 :)
declare function internal-nav:main() as xs:string
{
  let $persons-list := local:get-rdf-list("Person")
  let $works-list := local:get-rdf-list("Work")
  let $instances-list := local:get-rdf-list("Book")
  let $content :=
    element div {
        attribute id {"internal-area-container"},
        local:generic-panel("RDF-Personen", $persons-list),
        local:generic-panel("Rdf-Werke", $works-list),
        local:generic-panel("RDF-Instanzen", $instances-list),
        for $agg in $internal-nav:aggcollection
          let $filename := substring-after(base-uri($agg), "agg/")
          let $uri := substring-before($filename, ".")
          return
              if (matches($filename, "xml")
              and functx:is-value-in-sequence($uri, $internal-nav:relevant-aggs)) then
                let $doc := doc($config:sade-meta || $filename)
                let $title := replace($doc//tgmd:title/string(), " ", "-")
                let $xml-list := local:get-xml-list($agg)
                return local:generic-panel($title, $xml-list)
              else
                ()
    }
    return xmldb:store($config:app-root || "/templates/", "internal-area-overview.html", $content)
};

(:~
 : Gets an XHTML list of a type of RDF object. The types used in the BdN project
 : are persons, works (which means literary works) and instances of said works.
 :
 : @author Michelle Weidling
 : @param $type the type of RDF object to be rendered
 : @return element(ul) a list of files relating to the RDF type
 :)
declare function local:get-rdf-list($type as xs:string) as element(ul)
{
  let $rdfcollection := collection( $config:data-root || "/rdf" )
  let $rdfs := $rdfcollection//rdf:Description[1][rdf:type[matches(@rdf:resource, $type)]]
  let $uris :=
    for $rdf in $rdfs return
      substring-after($rdf/@rdf:about/string(), "textgridrep.de/")

  return local:make-list($uris)
};


(:~
 : Creates a list of the full URIs of XML files that belong to a aggregation.
 :
 : @author Michelle Weidling
 : @param $agg the current opened TextGrid aggregation
 : @return element(ul) a list with all XML files belonging to the aggregation
 :)
declare function local:get-xml-list($agg as item()*) as element(ul)
{
  let $xmls := $agg//*[@rdf:resource]/@rdf:resource/string()
  let $uris :=
    for $xml in $xmls return
      substring-after($xml, "textgrid:")

  return local:make-list($uris)
};


declare function local:get-xml-list-as-sequence($agg as item()*) as xs:string*
{
  let $xmls := $agg//*[@rdf:resource]/@rdf:resource/string()
  return
    for $xml in $xmls return
      substring-after($xml, "textgrid:")
};


(:~
 : Creates a list containing all XML files that belong to an aggregation.
 :
 : @author Michelle Weidling
 : @param $uris all URIs that are part of the aggregation as textgrid:uri.revision
 : @return element(ul)
 :)
declare function local:make-list($uris as xs:string+) as element(ul)
{
  <ul class="nav-textgrid">
  {
    for $uri in $uris
      let $meta := $internal-nav:metacollection//tgmd:object[matches(tgmd:generic/tgmd:generated/tgmd:textgridUri, $uri)]
      let $title := $meta//tgmd:title[1]/string()
      let $format := $meta//tgmd:format/string()
      where $title != ""
      order by upper-case($uri)
        return
            if (not(matches($meta//tgmd:format, "aggregation"))) then
              <li class="{replace($format, "[^a-z]", "")}" title="{$format}" id="{$uri}">{
                if ($format = "text/xml") then
                  element a {
                    attribute href {"book.html?file=" || $uri},
                    attribute target {"_blank"},
                    $title
                  }
                else
                  $title}
                </li>
            else
                let $agg := doc($config:sade-agg || $uri || ".xml")
                  let $xml-list := local:get-xml-list-as-sequence($agg)
                  return local:make-sub-aggregation($title, $xml-list)
  }
  </ul>
};


(:~
 : Creates a generic Bootstrap panel that is used to group the data into the
 : different types of RDF and aggregations.
 :
 : @author Michelle Weidling
 : @param $title the text that is displayed in the panel head
 : @param $content everything that goes into the panel body
 : @return element(div) the complete panel
 :)
declare function local:generic-panel($title as xs:string, $content as element(ul))
as element(div)
{
  let $id := replace($title, "[^a-zA-Z]", "")
  let $ref := "#" || $id
  return
    <div class="col-sm-4">
      <div class="panel panel-blue">
        <div class="panel-heading">
          {$title}
          <table class="right-aligned" style="display: inline;">
            <tr>
              <td>
                <a href="{$ref}" data-toggle="collapse" class="collapsed">
                  <i class="fa fa-caret-square-down pull-right" />
                </a>
              </td>
            </tr>
          </table>
        </div>
        <div class="panel-body collapse" id="{$id}">
        {$content}
        </div>
      </div>
    </div>
};

declare function local:make-sub-aggregation($title as xs:string, $uris as xs:string*)
as element(div)
{
  <div class="nav-sub-aggregation">
      <div>{$title}</div>
      <div class="list-group">
      {
      for $uri in $uris
          let $meta := $internal-nav:metacollection//tgmd:object[matches(tgmd:generic/tgmd:generated/tgmd:textgridUri, $uri)]
          let $title := $meta//tgmd:title/string()
          where $title != ""
          order by upper-case($uri)
            return
              element a {
                  attribute href {"book.html?file=" || $uri},
                  attribute target {"_blank"},
                  attribute id {$uri},
                  attribute class {"list-group-item list-group-item-action"},
                  $title
              }
      }
      </div>
  </div>
};

declare function internal-nav:display($node as node(), $model as map()*)
as element(div)*
{
  doc($config:app-root || "/templates/internal-area-overview.html")//div[@id = "internal-area-container"]/*
};
